import EESConnect.constants as constants
from tkinter import filedialog
import tkinter as tk
import shutil, os


class EESConnector:

    def __init__(self, ees_file_path=None, keep_refprop=False):

        self.__clear_files()
        self.__with_initialization = False
        self.__keep_refprop = keep_refprop
        self.__ees_file_path = None

        if ees_file_path is not None:
            self.ees_file_path = ees_file_path

    def __enter__(self):

        self.__with_initialization = True
        self.__move_REFPROP_DIR(move_away=True)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):

        self.quit()

    def calculate(self, input_list):

        if self.is_ready:

            self.__prepare_input(input_list)

        try:

            system_comand = "{} {}".format(constants.EES_PATH, constants.EES_MACRO)
            os.chdir(constants.WORKSPACE_DIR)
            os.system(system_comand)

        except:

            return None

        else:

            return self.__collect_output()

    def quit(self):

        self.__move_REFPROP_DIR(move_away=False)
        if self.is_ready:
            # This part delete the file used for data IO
            self.__clear_files()

    def __prepare_input(self, input_list):

        if type(input_list) == list:

            self.__write_input_file(input_list, "default")
            last_file_path = os.path.join(constants.WORKSPACE_DIR, "default" + constants.IO_FILE_EXTENSION)

        elif type(input_list) == dict:

            key = "1"
            for key in input_list.keys():

                self.__write_input_file(input_list[key], key)

            last_file_path = os.path.join(constants.WORKSPACE_DIR, key + constants.IO_FILE_EXTENSION)

        else:
            return

        shutil.copy(last_file_path, os.path.join(constants.WORKSPACE_DIR, constants.EES_INPUT_FILENAME))

    def __collect_output(self):

        return_dict = dict()

        for file in os.listdir(constants.WORKSPACE_DIR):

            if file.endswith(constants.IO_FILE_EXTENSION):

                key = file.strip(constants.IO_FILE_EXTENSION)

                return_dict.update({

                    key: self.__read_output_file(key)

                })

        if "default" in return_dict.keys():

            return return_dict["default"]

        else:

            return return_dict

    def __move_REFPROP_DIR(self, move_away):

        if move_away:

            if os.path.isdir(constants.EES_REFPROP_DIR) and not self.__keep_refprop:
                # This part of the code moves the EES-REFPROP plugin folder away from its correct location in order to
                # prevent the plugin from asking the user the position of the Refprop folder on each iteration.
                shutil.move(constants.EES_REFPROP_DIR, constants.EES_REFPROP_TMP_DIR)

        else:

            if os.path.isdir(constants.EES_REFPROP_TMP_DIR):
                # This part of the code moves the EES-REFPROP plugin folder back to its correct location in order to
                # restore the plugin functionality
                shutil.move(constants.EES_REFPROP_TMP_DIR, constants.EES_REFPROP_DIR)

    @staticmethod
    def __write_input_file(input_list, filename):

        string_to_write = str(input_list[0])

        for element in input_list[1:]:
            string_to_write += "\t" + str(element)

        string_to_write.replace(".", ",")
        with open(os.path.join(constants.WORKSPACE_DIR, filename + constants.IO_FILE_EXTENSION), "w") as f:
            f.write(string_to_write)

    @staticmethod
    def __read_output_file(filename):

        with open(os.path.join(constants.WORKSPACE_DIR, filename + constants.IO_FILE_EXTENSION), "r") as f:

            lines = f.readlines()

        return_list = list()

        for line in lines:

            for element in line.strip("\n").split("\t"):

                try:

                    return_list.append(float(element.replace(",", ".")))

                except:

                    return_list.append(element)

        return return_list

    @staticmethod
    def __clear_files():

        for file in os.listdir(constants.WORKSPACE_DIR):
            if file.endswith(constants.IO_FILE_EXTENSION) or file.endswith(".ees") or file.endswith(".dat") or file.endswith(".DAT"):
                os.remove(os.path.join(constants.WORKSPACE_DIR, file))

    def select_file(self):

        root = tk.Tk()
        root.withdraw()

        self.ees_file_path = filedialog.askopenfilename(title='select EES file')

        root.destroy()

    @property
    def ees_file_path(self):
        return self.__ees_file_path

    @property
    def is_ready(self):

        return self.__ees_file_path is not None

    @ees_file_path.setter
    def ees_file_path(self, ees_file_path):

        if os.path.isfile(ees_file_path):

            self.__ees_file_path = ees_file_path

            if os.path.isfile(constants.EES_RUN_FILENAME):
                os.remove(constants.EES_RUN_FILENAME)

            shutil.copy(ees_file_path, constants.EES_RUN_FILENAME)

    @classmethod
    def modify_ees_executable_path(cls):

        constants.retrieve_EES_path()


if __name__ == "__main__":

    with EESConnector() as connector:

        connector.select_file()
        # result = connector.calculate({
        #
        #     "1": ["air_ha", 300, 1013.25],
        #     "2": ["R22", 300, 1013.25],
        #     "3": ["R236fa", 300, 1013.25],
        #     "4": ["R134a", 300, 1013.25],
        #     "5": ["R236fa", 300, 1013.25],
        #     "6": ["R134a", 300, 1013.25],
        #     "7": ["R236fa", 300, 1013.25],
        #     "8": ["R134a", 300, 1013.25]
        #
        # })
        result = connector.calculate(["air_ha", 300, 1013.25])
        print(result)


from tkinter import filedialog
import tkinter as tk
import os


def retrieve_EES_path(failure_possible=True):
    root = tk.Tk()
    root.withdraw()
    __EES_PATH = filedialog.askopenfilename(

        title='select EES executable'

    )
    root.destroy()

    if os.path.isfile(__EES_PATH):

        with open(__EES_PATH_FILE, "w") as f:
            f.write(__EES_PATH)

        return __EES_PATH

    else:

        if not failure_possible:
            raise FileNotFoundError(
                "\n\nEES executable path must be provided!\n\n\t{} is not a suitable path\n\nExecution will stop!\n\n".format(
                    EES_PATH))

        return None


"""
In this module the path of the EES executable has to be retrieved. 
This can be done in two ways:

    If the "ees_path.dat" file exists, the code will read the path from there, otherwise the program will ask the user 
    to identify the path. In order to prevent the code from asking the user the path each import call, the selected path
    is written in the "ees_path.dat" file.
    
    the macro file is checked and generated as well 

"""

EES_PATH = None
ROOT_DIR = os.path.dirname(__file__)
__EES_PATH_FILE = os.path.join(ROOT_DIR, "ees_path.dat")

if os.path.isfile(__EES_PATH_FILE):

    with open(__EES_PATH_FILE) as f:
        lines = f.readlines()

    if os.path.isfile(str(lines[0])):
        EES_PATH = str(lines[0])

if EES_PATH is None:
    EES_PATH = retrieve_EES_path(failure_possible=False)

EES_DIR = os.path.dirname(EES_PATH)
EES_REFPROP_DIR = os.path.join(EES_DIR, "Userlib", "Userlib", "EES_REFPROP")
EES_REFPROP_TMP_DIR = os.path.join(EES_DIR, "EES_REFPROP")

"""
    
    MACRO FILE GENERATION:
    macrofile should be saved in the workspace directory where the EES file will be stored too
    the name that the ees file must have is 

"""

WORKSPACE_DIR = os.path.join(ROOT_DIR, "workspace")

if not os.path.isdir(WORKSPACE_DIR):
    os.mkdir(WORKSPACE_DIR)

EES_MACRO = os.path.join(WORKSPACE_DIR, "ees_run_macro.EMF")
EES_RUN_FILENAME = os.path.join(WORKSPACE_DIR, "ees_program.ees")
EES_INPUT_FILENAME = "ees_input.dat"
EES_OUTPUT_FILENAME = "ees_output.dat"
IO_FILE_EXTENSION = ".pyees"


def set_macro():

    __macro_text = (

        "ONERROR GOTO 10\n"

        "Open '{ees_filename}'\n"
        "filename$=GetFirstFile$(*{input_extension})\n"

        "if (filename$='') then GOTO 10\n"

        "repeat\n"

        "\t" + "{import_statement}\n"
        "\t" + "solve\n"
        "\t" + "{export_statement}\n"
        "\t" + "filename$=GetNextFile$\n"

        "until (filename$='')\n"

        "10:quit"

    ).format(

        workspace_dir=WORKSPACE_DIR,
        ees_filename=EES_RUN_FILENAME,
        input_extension=IO_FILE_EXTENSION,
        import_statement=__get_io_statement(get_import=True),
        export_statement=__get_io_statement(get_import=False)

    )

    with open(EES_MACRO, "w") as f:
        f.write(__macro_text)


def __get_io_statement(get_import):

    if get_import:

        return "delete '{ees_input}'\n\trename filename$ '{ees_input}'".format(

            ees_input=os.path.join(WORKSPACE_DIR, EES_INPUT_FILENAME)

        )

    else:

       return "rename '{ees_output}' filename$".format(

            ees_output=os.path.join(WORKSPACE_DIR, EES_OUTPUT_FILENAME)

       )


set_macro()
